const { genImport } = require("./utils");

function injectImport(
  file,
  api,
  { from, default: defaultImport, importList: _importList }
) {
  const j = api.jscodeshift;
  const importList = _importList ?? [];
  const root = j(file.source);

  const newImport = genImport(j, from, defaultImport, ...importList);
  root.nodes()[0].program.body.unshift(newImport);
  return root.toSource();
}

function injectFunctionUse(file, api, { fnName, usedName }) {
  const j = api.jscodeshift;
  const root = j(file.source);

  const renderNode = root.find(j.FunctionDeclaration, (node) => {
    if (node.id.name === fnName) {
      return true;
    }
  });

  let isPluginInstalled = false;

  if (defaultImport) {
    const defaultList = root.find(j.CallExpression, (node) => {
      if (
        node.arguments
        && node.arguments.length
        && j.Identifier.check(node.arguments[0])
        && node.arguments[0].name === usedName
      ) {
        return true;
      }
      return false;
    });

    isPluginInstalled = defaultList.nodes().length > 0;
  }


  if (!isPluginInstalled) {
    const newExpress = j.expressionStatement(
      j.callExpression(
        j.memberExpression(j.identifier("app"), j.identifier("use")),
        [j.identifier(usedName)]
      )
    );
    renderNode.nodes()[0].body.body.splice(2, 0, newExpress);
  }

  return root.toSource();
}

module.exports = {
  injectImport,
  injectFunctionUse,
};
